@@ -63,13 +63,15 @@ public class JSONParser {
   public static final int ALLOW_BACKSLASH_ESCAPING_ANY_CHARACTER = 1 << 2;
   public static final int ALLOW_UNQUOTED_KEYS                    = 1 << 3;
   public static final int ALLOW_UNQUOTED_STRING_VALUES           = 1 << 4;
-  /** ALLOW_EXTRA_COMMAS causes any nunber of extra commas in arrays and objects to be ignored
+  /** ALLOW_EXTRA_COMMAS causes any number of extra commas in arrays and objects to be ignored
    * Note that a trailing comma in [] would be [,] (hence calling the feature "trailing" commas
-   * isn't really correct.  Since trailing commas is fundamentally incompatible with any future
+   * is either limiting or misleading.  Since trailing commas is fundamentally incompatible with any future
    * "fill-in-missing-values-with-null", it was decided to extend this feature to handle any
    * number of extra commas.
    */
   public static final int ALLOW_EXTRA_COMMAS                     = 1 << 5;
+  public static final int ALLOW_MISSING_COLON_COMMA_BEFORE_OBJECT = 1 << 6;
+  public static final int OPTIONAL_OUTER_BRACES = 1 << 7;
 
   public static final int FLAGS_STRICT = 0;
   public static final int FLAGS_DEFAULT = ALLOW_COMMENTS | ALLOW_SINGLE_QUOTES | ALLOW_BACKSLASH_ESCAPING_ANY_CHARACTER | ALLOW_UNQUOTED_KEYS | ALLOW_UNQUOTED_STRING_VALUES | ALLOW_EXTRA_COMMAS;
@@ -101,18 +103,20 @@ public class JSONParser {
 
   private static final CharArr devNull = new NullCharArr();
 
-  int flags = FLAGS_DEFAULT;
+  protected int flags = FLAGS_DEFAULT;
 
-  final char[] buf;  // input buffer with JSON text in it
-  int start;         // current position in the buffer
-  int end;           // end position in the buffer (one past last valid index)
-  final Reader in;   // optional reader to obtain data from
-  boolean eof=false; // true if the end of the stream was reached.
-  long gpos;          // global position = gpos + start
+  protected final char[] buf;  // input buffer with JSON text in it
+  protected int start;         // current position in the buffer
+  protected int end;           // end position in the buffer (one past last valid index)
+  protected final Reader in;   // optional reader to obtain data from
+  protected boolean eof=false; // true if the end of the stream was reached.
+  protected long gpos;          // global position = gpos + start
 
-  int event;         // last event read
+  protected int event;         // last event read
 
-  int stringTerm;    // The terminator for the last string we read: single quote, double quote, or 0 for unterminated.
+  protected int stringTerm;    // The terminator for the last string we read: single quote, double quote, or 0 for unterminated.
+
+  protected boolean missingOpeningBrace = false;
 
   public JSONParser(Reader in) {
     this(in, new char[8192]);
@@ -308,7 +312,7 @@ public class JSONParser {
     }
   }
 
-  private int getCharExpected(int expected) throws IOException {
+  protected int getCharExpected(int expected) throws IOException {
    for(;;) {
       int ch = getChar();
       if (ch==expected) return expected;
@@ -359,7 +363,7 @@ public class JSONParser {
   }
 
 
-  private boolean matchBareWord(char[] arr) throws IOException {
+  protected boolean matchBareWord(char[] arr) throws IOException {
     for (int i=1; i<arr.length; i++) {
       int ch = getChar();
       if (ch != arr[i]) {
@@ -418,10 +422,10 @@ public class JSONParser {
   private String getContext() {
     String context = "";
     if (start>=0) {
-      context += " BEFORE='" + errEscape(Math.max(start-60,0), start+1) + "'";
+      context += " AFTER='" + errEscape(Math.max(start - 60, 0), start + 1) + "'";
     }
     if (start<end) {
-      context += " AFTER='" + errEscape(start+1, start+40) + "'";
+      context += " BEFORE='" + errEscape(start + 1, start + 40) + "'";
     }
     return context;
   }
@@ -749,7 +753,7 @@ public class JSONParser {
 
 
   // isName==true if this is a field name (as opposed to a value)
-  private void handleNonDoubleQuoteString(int ch, boolean isName) throws IOException {
+  protected void handleNonDoubleQuoteString(int ch, boolean isName) throws IOException {
     if (ch == '\'') {
       stringTerm = ch;
       if ((flags & ALLOW_SINGLE_QUOTES) == 0) {
@@ -1019,7 +1023,15 @@ public class JSONParser {
     outer: for(;;) {
       switch (state) {
         case 0:
-          return event = next(getChar());
+          event = next(getChar());
+          if (event == STRING && (flags & OPTIONAL_OUTER_BRACES) != 0) {
+            if (start > 0) start--;
+            missingOpeningBrace = true;
+            stringTerm = 0;
+            valstate = 0;
+            event = next('{');
+          }
+          return event;
         case DID_OBJSTART:
           ch = getCharExpected('"');
           if (ch == '}') {
@@ -1039,7 +1051,11 @@ public class JSONParser {
         case DID_MEMNAME:
           ch = getCharExpected(':');
           if (ch != ':') {
-            throw err("Expected key,value separator ':'");
+            if ((ch == '{' || ch == '[') && (flags & ALLOW_MISSING_COLON_COMMA_BEFORE_OBJECT) != 0) {
+              start--;
+            } else {
+              throw err("Expected key,value separator ':'");
+            }
           }
           state = DID_MEMVAL;  // set state first because it might be pushed...
           return event = next(getChar());
@@ -1049,7 +1065,13 @@ public class JSONParser {
             pop();
             return event = OBJECT_END;
           } else if (ch != ',') {
-            throw err("Expected ',' or '}'");
+            if ((flags & ALLOW_EXTRA_COMMAS) != 0 && (ch == '\'' || ch == '"' || Character.isLetter(ch))) {
+              start--;
+            } else if (missingOpeningBrace && ch == -1 && (flags & OPTIONAL_OUTER_BRACES) != 0) {
+              missingOpeningBrace = false;
+              pop();
+              return event = OBJECT_END;
+            } else throw err("Expected ',' or '}'");
           }
           ch = getCharExpected('"');
           if (ch == '"') {
@@ -1081,7 +1103,11 @@ public class JSONParser {
             pop();
             return event = ARRAY_END;
           } else {
-            throw err("Expected ',' or ']'");
+            if ((ch == '{' || ch == '[') && (flags & ALLOW_MISSING_COLON_COMMA_BEFORE_OBJECT) != 0) {
+              return event = next(ch);
+            } else {
+              throw err("Expected ',' or ']'");
+            }
           }
       }
     } // end for(;;)
